static char *addrChip2Base=0;
static unsigned chip2Size=0;

int epicsGetPhysicalMemsize()
{
    return(PHYS_MEM_SIZE + chip2Size);
}

void epicsUsrConfigInit()
{

    sysSetTTRegs(); /* Turn on TTRs for upper 1/2 of memory region */
    /*  Check for and initialize the second MEMC. */

    if (chip2Size>0) {
	memAddToPool(addrChip2Base, chip2Size);
	printf("  Added 0x%8.8X additional bytes to memory pool at %p\n",
	    chip2Size, addrChip2Base);
    }
}

#ifdef MEMC_BASE_ADRS
/******************************************************************************
 *
 * A routine that can init a second MEMC if present.  Called from task level,
 * so it is OK to use vxMemProbe()
 *
 ******************************************************************************/

int sysMemc2Init()
{
    char	ProbeValue = 0x0f;		/* MEMC reset code */
    char	cpuSpeed;
    char	bar2;
    char	rcr2;
    char	size1;
    char	size2;

#include "epicsGetCpuSpeed.c"

    /* Probe to see if the second MEMC is present */
    if (vxMemProbe((void *)MEMC2_ID, VX_WRITE, 1, &ProbeValue) != 0)
      return (-1);

    /* Config the second MEMC's clock speed setting */
    *MEMC2_BCR = cpuSpeed;

    size1 = 1 << (*MEMC_MCR & 0x07);
    size2 = 1 << (*MEMC2_MCR & 0x07);

    /* 
     * The second MEMC must be aligned to its own size.  Therefore it will
     * ALWAYS start at either the size of the first one (when the first one
     * is either the same size or larger then the second), or at the size
     * of the second one (when the second one is larger than the first one).
     */

    bar2 = (size1/size2) * size2;
    if (bar2 != size1)
      bar2 = size2;

    rcr2 = (bar2 & 3) << 6;
    bar2 >>= 2;

    *MEMC2_BAR  = bar2;					/* RAM address MS */
    *MEMC2_RCR  = rcr2;					/* RAM address LS */
    *MEMC2_RCR  = rcr2|(MEMC_RCR_RAMEN+MEMC_RCR_SWAIT);	/* Enable second RAM */

    /* Tell the caller where we put the RAM */
    addrChip2Base = (char *) ((*MEMC2_BAR << 24)|((*MEMC2_RCR & 0xc0) << 16));

    /* Tell the caller how much we got */
    chip2Size = (4 << (*MEMC2_MCR & 0x07)) << 20;

    return(0);
}
#endif

/*******************************************************************************
*
* A handy memory interface status reporting function.
*
*******************************************************************************/
int sysDumpVmeMap()
{


    /* Show A16 range */
    printf("\nBackplane viewed from CPU:\n");
    printf("  A16 Backplane:      0x0000 -     0xFFFF\n");
    printf("      Local:      0xFFFF0000 - 0xFFFFFFFF\n");
    printf("  A24 Backplane:    0x000000 -   0xFFFFFF\n");
    printf("      Local:      0xF0000000 - 0xF0FFFFFF\n");
    printf("  A32 Backplane:  0x%8.8lX - 0x%8.8lX\n",
		(unsigned long)(*VMECHIP2_LBSAR3) << 16,
		(unsigned long)(*VMECHIP2_LBSAR3) | 0x0000ffff);

    printf("\nCPU viewed from backplane:\n");
    printf("  A16 not mapped into CPU board\n");

    if (*VMECHIP2_VSAMSR & VSAMSR2_A24)
    {
	unsigned long LocalMask    = (unsigned long)(*VMECHIP2_VSATR2) << 16;
	unsigned long LocalLoAddr  = (unsigned long)(*VMECHIP2_VSATR2)
                                     & LocalMask;
	unsigned long VmeLoAddr    = (unsigned long)(*VMECHIP2_VSAR2) << 16;
	unsigned long VmeHiAddr    = (unsigned long)(*VMECHIP2_VSAR2)
                                     | 0x0000ffff;

	printf("  A24 Backplane:    0x%6.6lX -   0x%6.6lX\n",
            VmeLoAddr, VmeHiAddr);
	printf("      Local:      0x%8.8lX - 0x%8.8lX\n",
            LocalLoAddr, LocalLoAddr+(VmeHiAddr-VmeLoAddr));
    }
    else
    {
	printf("  A24 not mapped into CPU board\n");
    }
    {
	unsigned long	LocalMask    = (unsigned long)(*VMECHIP2_VSATR1) << 16;
	unsigned long	LocalLoAddr  = (unsigned long)(*VMECHIP2_VSATR1)
                                       & LocalMask;
	unsigned long	VmeLoAddr    = (unsigned long)(*VMECHIP2_VSAR1) << 16;
	unsigned long	VmeHiAddr    = (unsigned long)(*VMECHIP2_VSAR1)
                                       | 0x0000ffff;

	printf("  A32 Backplane:  0x%8.8lX - 0x%8.8lX\n", VmeLoAddr, VmeHiAddr);
	printf("      Local:      0x%8.8lX - 0x%8.8lX\n",
                 LocalLoAddr, LocalLoAddr+(VmeHiAddr-VmeLoAddr));
    }
    return(0);
}
/*******************************************************************************
*
* Inspect the VME memory interface mapping registers to perform the
* translation needed for a sysLocalToBusAdrs().
*
*******************************************************************************/
STATUS VmeChip2LocalToBusAdrs 
(
unsigned long	localAdrs, 
void			**pBusAdrs,
unsigned long	TranslationReg,
unsigned long	AddressReg
)
{
	unsigned long	LocalMask      = TranslationReg << 16;
	unsigned long	LocalLoAddr  = TranslationReg & LocalMask;
	unsigned long	VmeLoAddr    = AddressReg << 16;
	unsigned long	VmeHiAddr    = AddressReg | 0x0000ffff;


	if ((unsigned long)localAdrs < LocalLoAddr)
		return(ERROR);

	if ((unsigned long)localAdrs > LocalLoAddr+(VmeHiAddr-VmeLoAddr))
		return(ERROR);

	*pBusAdrs = (void *)(((unsigned long)localAdrs-LocalLoAddr)+VmeLoAddr);
	return (OK);
 
}
/*******************************************************************************
*
* sysLocalToBusAdrs - convert a local address to a bus address
*
* This routine gets the VMEbus address that accesses a specified local
* memory address.
*
* RETURNS: OK, or ERROR if the address space is unknown or the mapping is not
* possible.
*
* SEE ALSO: sysBusToLocalAdrs()
*/
 
STATUS sysLocalToBusAdrs
    (
    int adrsSpace,      /* bus address space in which busAdrs resides */
    char *localAdrs,    /* local address to convert                   */ 
    char **pBusAdrs     /* where to return bus address                */ 
    )
{
	STATUS	ret;

#if (LOCAL_MEM_BUS_A32 == 0)
	if ((unsigned long)localAdrs >= LOCAL_MEM_BUS_A32_SIZE)
#else
    if (((unsigned long)localAdrs < LOCAL_MEM_BUS_A32)  ||
        ((unsigned long)localAdrs >= LOCAL_MEM_BUS_A32 + LOCAL_MEM_BUS_A32_SIZE))
#endif
	{	/* This is off-board memory - just return local address */
		*pBusAdrs = localAdrs;
		return (OK);
	}

    switch (adrsSpace)
        {
	case VME_AM_SUP_SHORT_IO:
	case VME_AM_USR_SHORT_IO: /* No local map for A16 */
		return (ERROR);
 
	case VME_AM_STD_SUP_PGM:
	case VME_AM_STD_SUP_DATA:
	case VME_AM_STD_USR_PGM:
	case VME_AM_STD_USR_DATA:

	    /* Check to see if the A24 mapping is on */
		if ((*VMECHIP2_VSAMSR & VSAMSR2_A24) == 0)
			return(ERROR);

		ret = VmeChip2LocalToBusAdrs(
			(unsigned long)localAdrs, 
			(void **)pBusAdrs, 
			(unsigned long)(*VMECHIP2_VSATR2), 
			(unsigned long)(*VMECHIP2_VSAR2));
		return(ret);

	case VME_AM_EXT_SUP_PGM:
	case VME_AM_EXT_SUP_DATA:
	case VME_AM_EXT_USR_PGM:
	case VME_AM_EXT_USR_DATA:

		ret = VmeChip2LocalToBusAdrs(
			(unsigned long)localAdrs, 
			(void **)pBusAdrs, 
			(unsigned long)(*VMECHIP2_VSATR1), 
			(unsigned long)(*VMECHIP2_VSAR1));
		return(ret);

	default:
		return (ERROR);
	}
}
/******************************************************************************
 *
 * A24 memory management routines. (JRW)
 *
 * A user may call sysA24MapRam() to place a chunk of memory in the
 * A24 memory space.  When this is done, an aligned malloc is done
 * on the specified size (after rounding up to power of 2) to get
 * the memory to map into A24-space.  If the malloc succeeds, then
 * the chunk of memory is mapped into A24 using VSAMSR2, and a memory
 * pool is created for future use with malloc and free -style management
 * calls.
 *
 ******************************************************************************/

static PART_ID		a24DmaPartId = NULL;


STATUS sysA24MapRam(unsigned long size)
{
	unsigned long	mask;
	unsigned long	bsize;
	void			*mem;

	/* Make sure we only do this once!!! */
	if (a24DmaPartId != NULL)
		return(ERROR);

	if (size >= 0x00800000)
	{ /* Maximum size is 8MB clip it */
		size  = 0x00800000;
		bsize = 0x00800000;
		mask  = 0xffffff80;
	}
	else if (size <= 0x00010000)
	{ /* Minimum size is 64K, clamp it */
		size  = 0x00010000;
		bsize = 0x00010000;
		mask  = 0xffffffff;
	}
	else
	{ /* Round to power of 2 boundary */
		mask  = 0xffffffc0;
		bsize = 0x00400000;
		for (; (bsize!=0)&&((bsize&size)==0); bsize>>=1, mask>>=1);
		if (bsize != size)
		{ /* We are not on a 2^n boundary, round up */
			bsize <<= 1;
			mask  <<= 1;
		}
	}
	mask &= 0x0000ffff;
	/* size  = total size to map */
	/* bsize = physical window size to use to map it */

	/* Allocate some memory to build the A24 memory pool out of. */
	/* It MUST be power-2 aligned for the mapping reg stuff to work out. */
	if ((mem = memalign(bsize, size)) == NULL)
		return(ERROR);

	/* Build A24 memory pool */
	if ((a24DmaPartId = memPartCreate(mem, size)) == NULL)
	{
		free(mem);
		return(ERROR);
	}

	/* Starts at 0 and ends at bsize-1 in A24 */
	*VMECHIP2_VSAR2   = (bsize-1) & 0xffff0000;

	*VMECHIP2_VSATR2  = (unsigned long)mem | mask;

	/* Enable the A24 mapping */

	*VMECHIP2_VSAMSR |=   (VSAMSR2_SNP_WSD|
			 VSAMSR2_WP	|
			 VSAMSR2_SUP	|
			 VSAMSR2_USR	|
			 VSAMSR2_A24	|
			 VSAMSR2_D64	|
			 VSAMSR2_BLK	|
			 VSAMSR2_PGM	|
			 VSAMSR2_DAT);	/* all but A32 */

	return(OK);
}

void *sysA24Malloc(unsigned long size)
{
  /* Allocate some default chunk if we have not yet initialized */
  if (a24DmaPartId == NULL)
    if (sysA24MapRam(0) != OK)
     return(NULL);

  return(memPartAlloc(a24DmaPartId, size));
}

STATUS sysA24Free(char *pBlock)
{
  if (a24DmaPartId == NULL)
    return(ERROR);

  return(memPartFree(a24DmaPartId, pBlock));
}
/*******************************************************************************
*
* sysBusToLocalAdrs - convert a bus address to a local address
*
* This routine gets the local address that accesses a specified VMEbus
* memory address.
*
* RETURNS: OK, or ERROR if the address space is unknown or the mapping is not
* possible.
*
* SEE ALSO: sysLocalToBusAdrs()
*/

STATUS sysBusToLocalAdrs
    (
    int adrsSpace,      /* bus address space in which busAdrs resides */
    char *busAdrs,      /* bus address to convert                     */
    char **pLocalAdrs   /* where to return local address              */
    )
    {

	unsigned long	VmeLoAddr;
	unsigned long	VmeHiAddr;
	unsigned long	LocalMask;
	unsigned long	LocalLoAddr;

	switch (adrsSpace)
	{
	case VME_AM_SUP_SHORT_IO:
	case VME_AM_USR_SHORT_IO:
		if (busAdrs > (char *)0x0000ffff)
			return (ERROR);

		*pLocalAdrs = (char *) (0xffff0000 | (int)busAdrs);
		return (OK);

	case VME_AM_STD_SUP_PGM:
	case VME_AM_STD_SUP_DATA:
	case VME_AM_STD_USR_PGM:
	case VME_AM_STD_USR_DATA:
		if (busAdrs > (char *)0x00ffffff)
			return (ERROR);

		LocalMask    = (unsigned long)(*VMECHIP2_VSATR2) << 16;
		LocalLoAddr  = (unsigned long)(*VMECHIP2_VSATR2) & LocalMask;
		VmeLoAddr    = (unsigned long)(*VMECHIP2_VSAR2) << 16;
		VmeHiAddr    = (unsigned long)(*VMECHIP2_VSAR2) | 0x0000ffff;
		if ((*VMECHIP2_VSAMSR & VSAMSR2_A24)&&(busAdrs >= (char *)VmeLoAddr)&&(busAdrs <= (char *)VmeHiAddr))
		{	/* Access would loop back onto local memory... */
			unsigned long	l;
			l = (unsigned long)busAdrs;
			l -= VmeLoAddr;
			l += LocalLoAddr;
			*pLocalAdrs = (char *)l;
		}
		else
		{
			*pLocalAdrs = (char *)(busAdrs + 0xf0000000);
		}
		return (OK);

	case VME_AM_EXT_SUP_PGM:
	case VME_AM_EXT_SUP_DATA:
	case VME_AM_EXT_USR_PGM:
	case VME_AM_EXT_USR_DATA:

		VmeLoAddr = (unsigned long)(*VMECHIP2_LBSAR3) << 16;
		VmeHiAddr = (unsigned long)(*VMECHIP2_LBSAR3) | 0x0000ffff;
	
		if ((busAdrs < (char *)VmeLoAddr)||(busAdrs > (char *)VmeHiAddr))
		{	/* Check slave decoders to see if is translated in-bound mapping */ 
			/* If so, the VME address would represent a local address */
			unsigned long	l;

			VmeLoAddr    = (unsigned long)(*VMECHIP2_VSAR1) << 16;
			VmeHiAddr    = (unsigned long)(*VMECHIP2_VSAR1) | 0x0000ffff;
			if (((unsigned long)busAdrs < VmeLoAddr)||((unsigned long)busAdrs > VmeHiAddr))
				return(ERROR);

			LocalMask = (unsigned long)(*VMECHIP2_VSATR1) << 16;
			LocalLoAddr = (unsigned long)(*VMECHIP2_VSATR1) & LocalMask;

			l = (unsigned long)busAdrs;
			l &= ~LocalMask;
			l |= LocalLoAddr;
			*pLocalAdrs = (char *)l;
		}
		else
		{
			*pLocalAdrs = busAdrs;
		}
		return (OK);

	default:
		return (ERROR);
	}
}
