/* sysLib.c - Motorola MVME162,MVME162LX system-dependent library */

/* Copyright 1984-1996 Wind River Systems, Inc. */
#include "copyright_wrs.h"

/*
modification history
--------------------
01?,06apr98,jps  configure for EPICS
01l,05dec96,wlf  doc: cleanup.
01k,21nov96,dat  INCLUDE_MMU_FULL and INCLUDE_MMU_BASIC are mutually exclusive,
		 SPR 4042.
01j,04oct96,dat  added LOCAL_MEM_AUTOSIZE (#7288)
01i,20aug96,dat  added sysPhysMemTop(), PHYS_MEM_SIZE, fixed sysBusToLocalAdrs
		 (SPR 2557) and sysLocalToBusAdrs().
01h,15aug96,dds  added support for SCSI-2.
01g,28sep95,dat  new BSP revision id
01f,03aug95,myz  fixed the warning message
01e,08mar95,ms   moved serial configuration to sysSerial.c.
01d,24mar94,vin  mmu tagging in sysHwInit2() only if shared memory included.
01c,17mar94,vin  inhibited snooping on vmeChip2. Added code to mark the
		 shared memory pages as non cacheable in sysHwInit2().
01b,10mar94,dzb  Added support for 4 serial channel, MCECC, SRAM sizing.
01a,04jan93,ccc  created by modifying version 02b of mv167/sysLib.c.
*/

/*
DESCRIPTION
This library provides board-specific routines.  The chip drivers included are:
    
    mccTimer.c - MCchip Timer library
    vmeChip2Vme.c - VME Chip library
    nvRam.c - non-volatile RAM library
    nulVme.c -  null VMEbus library
    vmeChip2Vme.c - VMEchip2 library
    ncr710Lib - SCSI controller driver
    if_ei - ??? ??? network driver (`ei')

INCLUDE FILES: sysLib.h

SEE ALSO:
.pG "Configuration"
*/

#include "vxWorks.h"
#include "vme.h"
#include "memLib.h"
#include "cacheLib.h"
#include "sysLib.h"
#include "config.h"
#include "string.h"
#include "intLib.h"
#include "logLib.h"
#include "taskLib.h"
#include "vxLib.h"
#include "private/vmLibP.h"
#include "drv/scsi/ncr710.h"

#include "epicsConfig.h"
/* defines */

#define PHYS_MEM_SIZE ((int)sysPhysMemTop () - LOCAL_MEM_LOCAL_ADRS)

/* globals */

#ifndef	MV162_68EC040
PHYS_MEM_DESC sysPhysMemDesc [] =
    {
    /* adrs and length parameters must be page-aligned (multiples of 0x2000) */

    /* ram */
    {
    (void *) LOCAL_MEM_LOCAL_ADRS,
    (void *) LOCAL_MEM_LOCAL_ADRS,
    0x0,	/* set by sysHwInit, from sysPhysMemTop */
    VM_STATE_MASK_VALID	| VM_STATE_MASK_WRITABLE | VM_STATE_MASK_CACHEABLE,
    VM_STATE_VALID	| VM_STATE_WRITABLE	 | VM_STATE_CACHEABLE
    },

    /* rom */
    {
    (void *) ROM_BASE_ADRS,
    (void *) ROM_BASE_ADRS,
    0x400000,
    VM_STATE_MASK_VALID	| VM_STATE_MASK_WRITABLE | VM_STATE_MASK_CACHEABLE,
    VM_STATE_VALID	| VM_STATE_WRITABLE_NOT  | VM_STATE_CACHEABLE_NOT
    },

    /* sram */
    {
    (void *) 0xffe00000,
    (void *) 0xffe00000,
    0x20000,
    VM_STATE_MASK_VALID	| VM_STATE_MASK_WRITABLE | VM_STATE_MASK_CACHEABLE,
    VM_STATE_VALID	| VM_STATE_WRITABLE      | VM_STATE_CACHEABLE
    },

    /* local i/o devices */
    {
    (void *) 0xfff00000,
    (void *) 0xfff00000,
    0x40000,
    VM_STATE_MASK_VALID	| VM_STATE_MASK_WRITABLE | VM_STATE_MASK_CACHEABLE,
    VM_STATE_VALID	| VM_STATE_WRITABLE      | VM_STATE_CACHEABLE_NOT
    },

#ifdef	INCLUDE_VMECHIP2
    /* VMEchip2 */
    {
    (void *) 0xfff40000,
    (void *) 0xfff40000,
    0x2000,
    VM_STATE_MASK_VALID | VM_STATE_MASK_WRITABLE | VM_STATE_MASK_CACHEABLE,
    VM_STATE_VALID      | VM_STATE_WRITABLE      | VM_STATE_CACHEABLE_NOT
    },

    /* some a32 vme */
    {
    /*     (void *) 0x2000000, */
    /*     (void *) 0x2000000, */
    (void *) 0x80000000, 
    (void *) 0x80000000, 
    0x1000000,				/* 16MB */
    VM_STATE_MASK_VALID	| VM_STATE_MASK_WRITABLE | VM_STATE_MASK_CACHEABLE,
    VM_STATE_VALID	| VM_STATE_WRITABLE	 | VM_STATE_CACHEABLE_NOT
    },

    /* a24 vme */
    {
    (void *) 0xf0000000,
    (void *) 0xf0000000,
    0x1000000,				/* 16MB */
    VM_STATE_MASK_VALID	| VM_STATE_MASK_WRITABLE | VM_STATE_MASK_CACHEABLE,
    VM_STATE_VALID	| VM_STATE_WRITABLE      | VM_STATE_CACHEABLE_NOT
    },

    /* a16 vme */
    {
    (void *) 0xffff0000,
    (void *) 0xffff0000,
    0x10000,				/* 64KB */
    VM_STATE_MASK_VALID	| VM_STATE_MASK_WRITABLE | VM_STATE_MASK_CACHEABLE,
    VM_STATE_VALID	| VM_STATE_WRITABLE      | VM_STATE_CACHEABLE_NOT
    },
#endif	/* INCLUDE_VMECHIP2 */

    /* local i/o devices */
    {
    (void *) 0xfff42000,
    (void *) 0xfff42000,
    0xae000,
    VM_STATE_MASK_VALID | VM_STATE_MASK_WRITABLE | VM_STATE_MASK_CACHEABLE,
    VM_STATE_VALID      | VM_STATE_WRITABLE      | VM_STATE_CACHEABLE_NOT
    }
    };

int sysPhysMemDescNumEnt = NELEMENTS (sysPhysMemDesc);
#endif	/* MV162_68EC040 */

int   sysBus      = BUS;                /* system bus type (VME_BUS, etc)    */
int   sysCpu      = CPU;                /* system CPU type (MC680x0)         */
char *sysBootLine = BOOT_LINE_ADRS;	/* address of boot line              */
char *sysExcMsg   = EXC_MSG_ADRS;	/* catastrophic message area         */
int   sysFlags;				/* boot flags                        */
char  sysBootHost [BOOT_FIELD_LEN];	/* name of host from which we booted */
char  sysBootFile [BOOT_FIELD_LEN];	/* name of file from which we booted */

char *	sysPhysMemTop (void);		/* physical top of memory */

/* All 6 bytes are initialized in sysHwInit from BBRAM */
unsigned char eiEnetAddr [6] = { 0x08, 0x00, 0x3e, 0x00, 0x00, 0x00 };

/* Locals */

LOCAL int sysProcNum;			/* processor number of this CPU */

#include "timer/mccTimer.c"
#include "mem/nvRam.c"

#include "sysSerial.c"
#include "sysScsi.c"

#ifdef	INCLUDE_VMECHIP2		/* defined in config.h if included */
#include "vme/vmeChip2Vme.c"
#else	/* INCLUDE_VMECHIP2 */
#include "vme/nullVme.c"
#endif	/* INCLUDE_VMECHIP2 */

#ifdef	 INDUSTRY_PACK_C_FILE
#include INDUSTRY_PACK_C_FILE
#endif	 /* INDUSTRY_PACK_C_FILE */

/*******************************************************************************
*
* sysModel - return the model name of the CPU board
*
* This routine returns the model name of the CPU board.
*
* RETURNS: A pointer to the string "Motorola MVME162" or "Motorola MVME162LX".
*/

char *sysModel (void)
    {
#ifdef	MVME162LX
    return ("Motorola MVME162LX");
#else	/* MVME162LX */
    return ("Motorola MVME162");
#endif	/* MVME162LX */
    }

/*******************************************************************************
*
* sysBspRev - return the BSP version and revision number
*
* This routine returns a pointer to a BSP version and revision number, for
* example, 1.1/0. BSP_REV is concatenated to BSP_VERSION and returned.
*
* RETURNS: A pointer to the BSP version/revision string.
*/

char * sysBspRev (void)
    {
    return (BSP_VERSION BSP_REV);
    }

/*******************************************************************************
*
* sysHwInit - initialize the system hardware
*
* This routine initializes various features of the MVME162 and MVME162LX
* boards.  It sets up the control registers, initializes the Memory
* Controller ASIC (MCchip), disables the timers, and initializes the
* VMEchip2.  It is called from usrInit() in usrConfig.c.
*
* The sysProcNumSet() routine maps any local memory on the VMEbus.
*
* This routine must be called in supervisor mode.
*
* NOTE: This routine should not be called directly by the user application.
*
* RETURNS: N/A
*/

void sysHwInit (void)
    {
    int temp;   /* temp storage */

    /*
     * set up the MCchip.  This does the following:
     *   - Initialize the MCchip Interrupt Vector Base (upper 4 bits)
     *   - Disable and initialize the tick timer counter to zero
     *   - Disable SCC interrupts
     *   - Disable LANCE interrupts
     *   - Disable SCSI interrupts
     */

    *MCC_VBR = MCC_INT_VEC_BASE;
    *MCC_PARITY_ICR = PARITY_ICR_DIS;   /* disable parity error interrupt */

    /* fail LED on, and RESET switch enabled */

    *MCC_RESET_CR = RESET_CR_BDFLO | RESET_CR_RSWE;

    /* turn off watchdog timer */

    *MCC_WD_TIMER_CR = WD_TIMER_CR_DIS;
    *MCC_WD_TIMEOUT_REG = WD_TIMEOUT_REG_WD_16MS |
			  WD_TIMEOUT_REG_LB_256US;

    /* setup SRAM base address and enable */

    *MCC_SRAM_BASE_AR_HIGH = (char) (LOCAL_SRAM_LOCAL_ADRS >> 24);
    *MCC_SRAM_BASE_AR_LOW  = (char) (LOCAL_SRAM_LOCAL_ADRS >> 16);

    temp = (*MCC_DRAM_SRAM_OPTIONS & DRAM_SRAM_OPTIONS_SMASK) >> 3;
    if (temp == 0)				/* treat 128KB as 512KB */
	temp |= SRAM_SPACE_512K;
    *MCC_SRAM_SPACE_SIZE = temp | SRAM_SPACE_ENABLE; 

    /* setup MCECC chip(s) if ECC DRAM module is installed */

    if ((*MCC_DRAM_SPACE_SIZE == 6) && (MCECC_SCRUB_PERIOD > 0))
	{
	/* set the scrubber period */

	*MCECC_SPR_HI = (MCECC_SCRUB_PERIOD & 0xff00) >> 8;
	*MCECC_SPR_LO = MCECC_SCRUB_PERIOD & 0x00ff;
	*MCECC_STR = MCECC_STR_OFF_16 | MCECC_STR_ON_1;
	*MCECC_DCR = 0x0;

        *MCECC_SCR = MCECC_SCR_SCRBEN;		/* enable DRAM scrubber */
	}

    /* Initialize the Tick Timer Counters to zero */

    *MCC_TIMER4_CR	    = TIMER4_CR_DIS; /* make sure counters are off */
    *MCC_TIMER3_CR          = TIMER3_CR_DIS;
    *MCC_TIMER2_CR          = TIMER2_CR_DIS;
    *MCC_TIMER1_CR	    = TIMER1_CR_DIS;

    *MCC_TIMER1_CNT	    = 0x00;
    *MCC_TIMER2_CNT	    = 0x00;
    *MCC_TIMER3_CNT         = 0x00;
    *MCC_TIMER4_CNT         = 0x00;

    *MCC_TIMER1_CMP         = 0x00;
    *MCC_TIMER2_CMP         = 0x00;
    *MCC_TIMER3_CMP         = 0x00;
    *MCC_TIMER4_CMP         = 0x00;

    if (*MCC_VERSION_REG & 0x01)	/* check clock speed */
	{
        *MCC_PRESCALE_CLK_ADJ  = 256 - 32;	/* for 32 MHz board */
        *MCC_PRESCALE          = 256 - 32;
	*MCC_BUS_CLK_REG       = 32;
	}
    else
	{
	*MCC_PRESCALE_CLK_ADJ  = 256 - 25;	/* for 25 MHz board */
	*MCC_PRESCALE          = 256 - 25;
	*MCC_BUS_CLK_REG       = 25;
	}

    *MCC_T4_IRQ_CR	    = T4_IRQ_CR_DIS; /* make sure interrupts are off */
    *MCC_T3_IRQ_CR	    = T3_IRQ_CR_DIS; /* will be enabled later if used */
    *MCC_T2_IRQ_CR          = T2_IRQ_CR_DIS;
    *MCC_T1_IRQ_CR          = T1_IRQ_CR_DIS;

    /* Set up the LANC Interrupt Control Registers
     * For now leave the Interrupts disabled to
     * prevent the interrupts from causing problems until
     * we are ready to use them in the LANC drive
     */

    *MCC_LANC_IRQ_CR	= LANC_IRQ_CR_DIS;	/* Disable Interrupts for now */
    *MCC_LANC_BEICR	= LANC_BEICR_DIS |	/* Disable Bus Error Ints */
			  LANC_BEICR_SINK_DATA;

    /* Set up the SCSI Interrupt Control Registers
     * For now leave the Interrupts disabled to
     * prevent the interrupts from causing problems until
     * we are ready to use them in the SCSI driver
     */

    *MCC_SCSI_IRQ_CR	= SCSI_IRQ_CR_DIS;	/* Disable Interrupts for now */

    /* now see if there is an Industry Pack driver to call */

#ifdef	INDUSTRY_PACK_C_FILE
    sysIndPackInit ();		/* initialization specific to the IP module */
#endif	/* INDUSTRY_PACK_C_FILE */

#ifdef	INCLUDE_VMECHIP2
    /* disable VME access to on-board ram and VMEchip slave write posting.
     * (VME access is later enabled for processor 0 in sysProcNumSet.)
     *
     * Set-up VMEbus access as follows:
     *
     *   SHORT_IO	0xffff0000   - 0xffffffff
     *   STD (A24/D16)  0xf0000000   - 0xf0ffffff
     *   EXT (A32/D32)  sysPhysMemTop- 0xefffffff
     */


    /* set VMEbus short access and enable map */
    /* set VMEbus std access and enable map */
    /* set VMEbus ext access and enable map */
    /* make sure map 2-4 is disabled */
#include "epicsMapVme.c"


    /* set VMEbus global timeout to 256us
     * set VMEbus access timeout to 32ms
     * set local bus timeout to 256us
     * set watchdog timeout to 16ms
     * and set prescaler register
     */

    if (*MCC_VERSION_REG & 1)
	{
        *VMECHIP2_TIMEOUTCR	    = (TIMEOUTCR_VGTO_256US |
				       TIMEOUTCR_VATO_32MS  |
				       TIMEOUTCR_LBTO_256US |
				       TIMEOUTCR_WDTO_16MS) |
				       (256 - 33);	/* for 33MHz board */
        }
    else
	{
        *VMECHIP2_TIMEOUTCR         = (TIMEOUTCR_VGTO_256US |
				       TIMEOUTCR_VATO_32MS  |
				       TIMEOUTCR_LBTO_256US |
				       TIMEOUTCR_WDTO_16MS) |
				       (256 - 25);	/* for 25MHz board */
	}

    /* all VMEbus vectors with 1:1 mapping */

    *VMECHIP2_ILR4		= (ILR4_VIRQ7_LEVEL7	|
				   ILR4_VIRQ6_LEVEL6	|
				   ILR4_VIRQ5_LEVEL5	|
				   ILR4_VIRQ4_LEVEL4	|
				   ILR4_VIRQ3_LEVEL3	|
				   ILR4_VIRQ2_LEVEL2	|
				   ILR4_VIRQ1_LEVEL1);

    /*
     * setup VMEbus requester control:
     * priority arbiter, release when done, level 3.
     */

    *VMECHIP2_DMACR1		= (DMACR1_PRIORITY	|
				   DMACR1_LVRWD		|
				   DMACR1_LVREQ_L3);

    /* master enable chip interrupts */

    *VMECHIP2_IOCR		= (IOCR_MEIN			|
				   (UTIL_INT_VEC_BASE0 << 24)	|
				   (UTIL_INT_VEC_BASE1 << 20));

    /* turn LED off, disable watchdog, disable timers */

    temp = *VMECHIP2_TIMERCR & 0x0000ffff;
    *VMECHIP2_TIMERCR = temp |
			TIMERCR_RSWE |
			TIMERCR_WDDIS;

    *VMECHIP2_TIMERCR &= ~(TIMERCR_TT2_EN | TIMERCR_TT1_EN);
    *VMECHIP2_LBIER = 0x00;	/* make sure all interrupts are disabled */
    *VMECHIP2_MISCCR = MISCCR_DISSRAM;	/* disable SRAM (MCChip does SRAM) */
#endif	/* INCLUDE_VMECHIP2 */

    *MCC_GCR = GCR_MIEN_ON;

    /* now setup serial device descriptor */

    sysSerialHwInit();

    /*
     * Extract the ethernet address out of non-volatile RAM.
     * Motorola has the ethernet address in BBRAM.  If the address
     * is not in BBRAM you must reset it to the address located
     * on a label on the P2 connector.
     */

    eiEnetAddr [0] = (UCHAR)BB_ENET [0];
    eiEnetAddr [1] = (UCHAR)BB_ENET [1];
    eiEnetAddr [2] = (UCHAR)BB_ENET [2];
    eiEnetAddr [3] = (UCHAR)BB_ENET [3];
    eiEnetAddr [4] = (UCHAR)BB_ENET [4];
    eiEnetAddr [5] = (UCHAR)BB_ENET [5];

    /* now let's turn off the FAIL LED */

    *MCC_RESET_CR &= ~(RESET_CR_BDFLO);

    /*
     * If mmu tables are used, this is where we dynamically
     * update the entry describing main memory, using sysPhysMemTop().
     * We must call sysPhysMemTop() here to do the memory autosizing
     * if available.
     */

    #include "epicsMemInit.c"
    }

/*******************************************************************************
*
* sysPhysMemTop - get the address of the top of physical memory
*
* This routine returns the address of the first missing byte of memory,
* which indicates the top of memory.
*
* Normally, the user specifies the amount of physical memory with the
* macro LOCAL_MEM_SIZE in config.h.  BSPs that support run-time
* memory sizing do so only if the macro LOCAL_MEM_AUTOSIZE is defined.
* If not defined, then LOCAL_MEM_SIZE is assumed to be, and must be, the
* true size of physical memory.
*
* NOTE: Do no adjust LOCAL_MEM_SIZE to reserve memory for application
* use.  See sysMemTop() for more information on reserving memory.
*
* RETURNS: The address of the top of physical memory.
*
* SEE ALSO: sysMemTop()
*/

char * sysPhysMemTop (void)
    {
    static char * memTop = NULL;

    if (memTop == NULL)
	{
#ifdef LOCAL_MEM_AUTOSIZE
	/* If we could do autosizing, this is the place for it */
        #include "epicsMCCAutoSize.c"  /* We can do it! */
#else

	/* Don't do autosizing, if size is given */
	memTop = (char *)(LOCAL_MEM_LOCAL_ADRS + LOCAL_MEM_SIZE);

#endif /*LOCAL_MEM_AUTOSIZE*/
	}
    
    return memTop;
    }

/*******************************************************************************
*
* sysMemTop - get the address of the top of VxWorks memory
*
* This routine returns a pointer to the first byte of memory not
* controlled or used by VxWorks.
*
* The user can reserve memory space by defining the macro USER_RESERVED_MEM
* in config.h.  This routine returns the address of the reserved memory
* area.  The value of USER_RESERVED_MEM is in bytes.
*
* RETURNS: The address of the top of VxWorks memory.
*/

char *sysMemTop (void)
    {
    static char * memTop = NULL;

    if (memTop == NULL)
	{
	memTop = sysPhysMemTop () - USER_RESERVED_MEM;
	}

    return memTop;
    }

/*******************************************************************************
*
* sysAbortInt - handle ABORT interrupt
*
* This routine handles the ABORT switch interrupt
*
* RETURNS: N/A
*/

LOCAL void sysAbortInt (void)
    {
    *MCC_RESET_CR |= RESET_CR_BDFLO;    /* turn on LED */
    *MCC_ABORT_ICR |= ABORT_ICR_ICLR;   /* clear interrupt */

    sysToMonitor (BOOT_NO_AUTOBOOT);
    }

/*******************************************************************************
*
* sysHwInit2 - initialize additional system hardware
*
* This routine connects system interrupt vectors and configures any 
* required features not configured by sysHwInit().
*
* RETURNS: N/A
*/

void sysHwInit2 (void)
    {
    static BOOL configured = FALSE;

    if (!configured)
        {
        /* connect system clock interrupt */
        (void) intConnect (INUM_TO_IVEC (INT_VEC_CLOCK), sysClkInt, NULL);

        /* connect and enable abort switch interrupt */
        (void) intConnect (INUM_TO_IVEC (INT_VEC_ABORT), sysAbortInt, NULL);

        *MCC_ABORT_ICR |= ABORT_ICR_IEN; /* enable the abort button interrupt */
        *MCC_ABORT_ICR |= ABORT_IRQ_LEVEL;

        /* connect auxiliary clock interrupt */
        (void) intConnect (INUM_TO_IVEC(INT_VEC_AUX_CLOCK), sysAuxClkInt, NULL);

        /* connect serial interrupts */

	sysSerialHwInit2();

    /* configure the shared memory area as non-cacheable */

#ifdef INCLUDE_SM_NET		/* only if shared memory included */
#if !SM_OFF_BOARD		/* if shared memory is onboard */

#ifdef INCLUDE_MMU_FULL                 /* unbundled mmu product */
    (void) vmStateSet (NULL, (void *)SM_MEM_ADRS,
                       SM_MEM_SIZE + SM_OBJ_MEM_SIZE,
                       VM_STATE_MASK_CACHEABLE,
                       VM_STATE_CACHEABLE_NOT);
#elif defined(INCLUDE_MMU_BASIC)	/* if basic mmu support */
    (void) vmBaseStateSet (NULL, (void *)SM_MEM_ADRS,
                           SM_MEM_SIZE + SM_OBJ_MEM_SIZE,
                           VM_STATE_MASK_CACHEABLE,
                           VM_STATE_CACHEABLE_NOT);
#endif /* INCLUDE_MMU_BASIC */

#endif /* SM_OFF_BOARD */
#endif /* INCLUDE_SM_NET */

        configured = TRUE;
	}
    }

/*******************************************************************************
*
* sysToMonitor - transfer control to the ROM monitor
*
* This routine transfers control to the ROM monitor.  Normally, it is called
* only by reboot()--which services ^X--and by bus errors at interrupt level.
* However, in some circumstances, the user may wish to introduce a
* <startType> to enable special boot ROM facilities.
*
* RETURNS: OK, if there is a return from the ROM monitor.
*/

STATUS sysToMonitor
    (
    int startType     /* parameter passed to ROM to tell it how to boot */
    )
    {
    /* this is an offset from romInit to the entry for a warm
     * start.  If romInit is changed this may need to be modified
     */

    FUNCPTR pRom = (FUNCPTR) (ROM_TEXT_ADRS + 8);

    sysSerialReset ();

    /* disable the mmu */

    VM_ENABLE(FALSE);

    (*pRom) (startType);

    return (OK);	/* in case we ever continue from ROM monitor */
    }

/******************************************************************************
*
* sysProcNumGet - get the processor number
*
* This routine returns the processor number for the CPU board, which is set
* with sysProcNumSet().
*
* RETURNS: The processor number for the CPU board.
*
* SEE ALSO: sysProcNumSet()
*/

int sysProcNumGet (void)
    {
    return (sysProcNum);
    }

/******************************************************************************
*
* sysProcNumSet - set the processor number
*
* This routine sets the processor number for the CPU board.  Processor numbers
* should be unique on a single backplane.
*
* NOTE: This routine enables access from the VMEbus to the MVME162's local
* memory only for processor 0.  (VxWorks only requires mapping for
* processor 0, and then only for backplane networks.)  If the processor
* number is not 0, but the user wants to allow VMEbus access to local memory,
* change the "if (procNum == 0)" condition below.
*
* This routine also sets the VMEchip GCSR address based on the processor
* number.  This address can be specified in the short I/O space from 0x00 to
* 0xe0 in increments of 0x10.  This can allow up to 15 CPUs in the same cage
* to form a backplane network using signal interrupts to coordinate their
* activities.
*
* RETURNS: N/A
*
* SEE ALSO: sysProcNumGet()
*/

#include "epicsSysProcNumSet.c"

#ifdef	INCLUDE_VMECHIP2
/******************************************************************************
*
* sysBusTas - test and set a location across the bus
*
* This routine performs a test-and-set instruction across a backplane.
*
* NOTE: If a problem with TAS instruction occurs, contact Motorola; some
* older versions of boards have problems with the RMW cycle.  There is
* a PAL change to take care of this problem.
*
* RETURNS: TRUE if the value had not been set but is now, or FALSE if the
* value was set already.
*
* SEE ALSO: vxTas()
*/

BOOL sysBusTas
    (
    char *adrs		/* address to be tested and set */
    )
    {
    return (vxTas (adrs));
    }
#endif	/* INCLUDE_VMECHIP2 */

/* miscellaneous support routines */

#ifdef	INCLUDE_EI
/* 82596 Ethernet chip support rotuines */
/*******************************************************************************
*
* sys596Init - performs any additional target specific initialization
*
* NOMANUAL
*/

void sys596Init
    (
    int unit		/* ignored on MVME162 and MVME162LX */
    )
    {
    /* None required */
    }

/*******************************************************************************
*
* sys596IntDisable - performs any additional target specific interrupt disabling
*
* NOMANUAL
*/

void sys596IntDisable
    (
    int unit		/* ignored on MVME162 and MVME162LX */
    )
    {
    int level;

    /* The CPU interrupt level is raised prior to modifying the bit in the
     * MCC device.  This is done to avoid a known hardware problem in the
     * MCC device that can cause spurious interrupts.
     */

    level = intLock ();
    *MCC_LANC_IRQ_CR &= ~LANC_IRQ_CR_IEN;     /* Clear the enable bit only */
    intUnlock (level);
    }

/*******************************************************************************
*
* sys596IntEnable - performs any additional target specific interrupt enabling
*
* This routine enables interrupts for the on-board LAN chip.  LANC interrupts
* are controlled by the MCC chip.
*
* NOMANUAL
*/

void sys596IntEnable
    (
    int unit		/* ignored on MVME162 and MVME162LX */
    )
    {
    *MCC_LANC_IRQ_CR =
                        (
                        LANC_IRQ_CR_IEN      |    /* set enable bit */
                        LANC_IRQ_CR_EDGE     |    /* must be edge trigger */
                        LANC_IRQ_CR_LOW_HIGH |    /* high-->low edge */
                        LANC_IRQ_LEVEL            /* priority level */
                        );
    }

/*******************************************************************************
*
* sys596IntAck - performs any additional target specific interrupt acknowledge
*
* NOMANUAL
*/

void sys596IntAck
    (
    int unit		/* ignored on MVME162 and MVME162LX */
    )
    {
    *MCC_LANC_IRQ_CR |= LANC_IRQ_CR_ICLR;
    }

/*******************************************************************************
*
* sys596Port - writes a command to the 82596 device PORT location
*
* There are 4 commands the device handles.
*
* NOMANUAL
*/

void sys596Port
    (
    int unit,		/* ignored on MVME162 and MVME162LX      */
    int cmd,		/* the command to write                  */
    UINT32 addr		/* address or NULL if PORT_RESET command */
    )
    {
    FAST UINT v1;
    FAST UINT value = (cmd & 0x3) | addr;

    v1 = ((value << 16) & 0xffff0000) | (value & 0x0000ffff);

    *I82596_PORT = v1;

    v1 = ((value >> 16) & 0x0000ffff) | (value & 0xffff0000);

    *I82596_PORT = v1;
    }

/*******************************************************************************
*
* sys596ChanAtn - assert the Channel Attention signal to the 82596 device
*
* NOMANUAL
*/

void sys596ChanAtn
    (
    int unit		/* ignored on MVME162 and MVME162LX */
    )
    {
    *I82596_CONTROL = 1;		/* write to the addr; data is ignored */
    }

/*******************************************************************************
*
* sysEnetAddrGet - get the ethernet address
*
* RETURNS: OK, always.
*
* NOMANUAL
*/

STATUS sysEnetAddrGet
    (
    int unit,		/* ignored on MVME162 and MVME162LX */
    UINT8 *addr
    )
    {
    bcopy ((char *)eiEnetAddr, (char *)addr, sizeof (eiEnetAddr));

    return (OK);
    }
#endif	/* INCLUDE_EI */

#include "epicsSyslib.c"

