/* sysScsi.c - Motorola MVME162 SCSI-2 modifications for sysLib.c */

/* Copyright 1984-1996 Wind River Systems, Inc. */
#include "copyright_wrs.h"

/*
modification history
--------------------
01f,05dec96,wlf  doc: cleanup.
01e,10oct96,dat  fixed warnings for SCSI-1 library
01d,20sep96,dat  added sysScsiConfig()
01c,19aug96,dds  added scsiCacheSnoopEnable support.
01b,15aug96,dds  added support for SCSI-2.
01a,18apr94,ccc  modified sysScsiInit() and support routines for SCSI-2.
*/

/* 
Description

This file contains the sysScsiInit() and related routines necessary for
the SCSI-2 component upgrade.  These routines and any defines in this
file will be incorporated into the mv162 BSP when the SCSI-2 support
is incorporated into VxWorks.
*/

#ifdef	INCLUDE_SCSI

#ifndef INCLUDE_SCSI2

#define	NV_SCSI_ADRS	(NV_RAM_ADRS + 0x1f35)	/* offset for SCSI bus ID */

/*******************************************************************************
*
* sysScsiInit - initialize an NCR 53C710 SCSI chip
*
* This routine creates and initializes an SIOP structure, enabling use of the
* on-board SCSI port.  It connects the proper interrupt service routine
* to the desired vector and enables the interrupt.
*
* RETURNS: OK, or ERROR if the SIOP structure cannot be created, the
* controller cannot be initialized, or the interrupt service routine cannot
* be connected.
*/
 
STATUS sysScsiInit (void)
    {
    /* Local structure with a prefill for ncr710SetHwRegister */
    static NCR710_HW_REGS hwRegs = MV162_SIOP_HW_REGS;
    UINT8 scsiBusId = 7;	/* read from NVRAM for SCSI bus ID */

    /* see if NVRAM location has an ASCII value for the SCSI bus ID */

    if (((UINT8) *(NV_SCSI_ADRS) & 0xf8) == 0x30)
        scsiBusId = (UINT8) *(NV_SCSI_ADRS) & 0x07;	/* read SCSI bus ID */
 
    if ((pSysScsiCtrl = (SCSI_CTRL *)ncr710CtrlCreate (MV162_SIOP_BASE_ADRS,
                                                       MV162_SIOP_FREQ
                                                       )) == NULL)
        {
        return (ERROR);
        }
 
    /* connect the SCSI controller's interrupt service routine */
 
    if (intConnect (INUM_TO_IVEC (INT_VEC_SCSI),
                    ncr710Intr, (int) pSysScsiCtrl) == ERROR)
        {
        return (ERROR);
        }
 
    /*
     * Set the good value in the registers of the SIOP coupled
     * with the hardware implementation
     * NO MUX HOST BUS/NO BURST ACCES/SNOOP :DEFAULT/ENABLE SYNC HOST BUS/
     * BURST SIZE DEFAULT/OTHER :DEFAULT CONFIGURATION
     */
 
    if (ncr710SetHwRegister ((NCR_710_SCSI_CTRL *)pSysScsiCtrl, &hwRegs) 
	== ERROR)
        return(ERROR);
 
    /* initialize SCSI controller with default parameters (user tuneable) */
 
    if (ncr710CtrlInit ((NCR_710_SCSI_CTRL *) pSysScsiCtrl, 
			scsiBusId, NONE) == ERROR)
        return (ERROR);
 
    /*
     * clear status enable the SIOP interrupt
     * don't enable MCchip scsi interrupt before ncr710CtrlInit
     */
 
   *MCC_SCSI_ERR_SR = SCSI_ERR_SR_SCLR;
   *MCC_SCSI_IRQ_CR = (SCSI_IRQ_CR_IEN |
                       SCSI_IRQ_LEVEL);
 
    return (OK);
    }

#else /* INCLUDE_SCSI2 */

#include "tapeFsLib.h"

/*******************************************************************************
*
* sysScsiInit - initialize NCR 710 SCSI chip
*
* This routine creates and initializes an SIOP structure, enabling use of the
* on-board SCSI port.  It also connects the proper interrupt service routine
* to the desired vector, and enables the interrupt at the desired level.
*
* RETURNS: OK, or ERROR if the control structure is not created or the
* interrupt service routine cannot be connected to the interrupt.
*/

STATUS sysScsiInit ()

    {
    /* Local structure with a prefill for ncr710SetHwRegister */
    static NCR710_HW_REGS hwRegs = MV162_SIOP_HW_REGS;

    if ((pSysScsiCtrl = (SCSI_CTRL *)ncr710CtrlCreateScsi2
                                (MV162_SIOP_BASE_ADRS, MV162_SIOP_FREQ
                                                       )) == NULL)
        {
        return (ERROR);
        }

    /* connect the SCSI controller's interrupt service routine */

    if (intConnect (INUM_TO_IVEC (INT_VEC_SCSI),
                    ncr710IntrScsi2, (int) pSysScsiCtrl) == ERROR)
        {
        return (ERROR);
        }

    /*
     * Set the good value in the registers of the SIOP coupled
     * with the hardware implementation
     * NO MUX HOST BUS/NO BURST ACCES/SNOOP :DEFAULT/ENABLE SYNC HOST BUS/
     * BURST SIZE DEFAULT/OTHER :DEFAULT CONFIGURATION
     */

    if (ncr710SetHwRegisterScsi2 ((NCR_710_SCSI_CTRL *)pSysScsiCtrl, &hwRegs)
        == ERROR)
        return(ERROR);


    /* initialize SCSI controller with default parameters (user tuneable) */

    if (ncr710CtrlInitScsi2 ((NCR_710_SCSI_CTRL *)pSysScsiCtrl,
                        SCSI_DEF_CTRL_BUS_ID, NONE) == ERROR)
        return (ERROR);

#if (USER_D_CACHE_MODE &  CACHE_SNOOP_ENABLE)

    scsiCacheSnoopEnable ((SCSI_CTRL *) pSysScsiCtrl);

#else

    scsiCacheSnoopDisable ((SCSI_CTRL *) pSysScsiCtrl);

#endif

    /*
     * clear status enable the SIOP interrupt
     * don't enable pcc scsi interrupt before ncr710CtrlInit
     */

   *MCC_SCSI_ERR_SR = SCSI_ERR_SR_SCLR ;
   *MCC_SCSI_IRQ_CR = (SCSI_IRQ_CR_IEN |
                        SCSI_IRQ_LEVEL);

#ifdef INCLUDE_TAPEFS
    tapeFsInit ();                      /* initialise tapeFs */
#endif

    return (OK);
    }

#endif /* INCLUDE_SCSI2 */

/* Data for example code in sysScsiConfig, modify as needed */

SCSI_PHYS_DEV *	pSpd20;
SCSI_PHYS_DEV *	pSpd31;         /* SCSI_PHYS_DEV ptrs (suffix == ID, LUN) */
SCSI_PHYS_DEV *	pSpd40;

BLK_DEV *	pSbd0;
BLK_DEV *	pSbd1;
BLK_DEV *	pSbd2;          /* SCSI_BLK_DEV ptrs for Winchester */
BLK_DEV *	pSbdFloppy;     /* ptr to SCSI floppy block device */

#ifdef INCLUDE_SCSI2
SEQ_DEV *	pSd0;
TAPE_CONFIG *	pTapeConfig;
#endif /* INCLUDE_SCSI2 */

/*******************************************************************************
*
* sysScsiConfig - system SCSI configuration
*
* This is an example SCSI configuration routine.
*
* Most of the code found here is an example of how to declare a SCSI
* peripheral configuration.  You must edit this routine to reflect the
* actual configuration of your SCSI bus.
* This example can also be found in src/config/usrScsi.c.
*
* If you are just getting started, you can test your hardware configuration
* by defining SCSI_AUTO_CONFIG in config.h, which will probe the bus and
* display all devices found.  No device should have the same SCSI bus ID as
* your VxWorks SCSI port (default = 7), or the same as any other device.
* Check for proper bus termination.
*
* There are three configuration examples here that 
* demonstrate configuration of a SCSI hard disk (any type), an OMTI 3500 floppy 
* disk, and a tape drive (any type).
*
* The hard disk is divided into two 32-Mbyte partitions and a third
* partition with the remainder of the disk.  The first partition is initialized
* as a dosFs device.  The second and third partitions are initialized as
* rt11Fs devices, each with 256 directory entries.
*
* It is recommended that the first partition (BLK_DEV) on a block device be
* a dosFs device, if the intention is eventually to boot VxWorks from the
* device.  This will simplify the task considerably.
*
* The floppy, since it is a removable medium device, is allowed to have only a
* single partition, and dosFs is the file system of choice for this device,
* since it facilitates media compatibility with IBM PC machines.
*
* While the hard disk configuration is fairly straightforward, the floppy
* setup in this example is a bit intricate.  Note that the
* scsiPhysDevCreate() call is issued twice.  The first time is merely to get
* a "handle" to pass to the scsiModeSelect() function, since the default
* media type is sometimes inappropriate (in the case of generic
* SCSI-to-floppy cards).  After the hardware is correctly configured, the
* handle is discarded via the scsiPhysDevDelete() call, after which a
* second scsiPhysDevCreate() call correctly configures the peripheral.
* (Before the scsiModeSelect() call, the configuration information was
* incorrect.) Also note that following the scsiBlkDevCreate() call, the
* correct values for <sectorsPerTrack> and <nHeads> must be set via the
* scsiBlkDevInit() call.  This is necessary for IBM PC compatibility.
* Similarly, the tape configuration is also a little involved because 
* certain device parameters need to turned off within VxWorks and the
* tape fixed block size needs to be defined, assuming that the tape
* supports fixed blocks.
*
* The last parameter to the dosFsDevInit() call is a pointer to a
* DOS_VOL_CONFIG structure.  By specifying NULL, you are asking
* dosFsDevInit() to read this information off the disk in the drive.  This
* may fail if no disk is present or if the disk has no valid dosFs
* directory.  Should this be the case, you can use the dosFsMkfs() command to
* create a new directory on a disk.  This routine uses default parameters
* (see dosFsLib) that may not be suitable for your application, in which
* case you should use dosFsDevInit() with a pointer to a valid DOS_VOL_CONFIG
* structure that you have created and initialized.  If dosFsDevInit() is
* used, a diskInit() call should be made to write a new directory on the
* disk, if the disk is blank or disposable.
*
* NOTE
* The variable <pSbdFloppy> is global to allow the above calls to be
* made from the VxWorks shell, i.e.:
* .CS
*     -> dosFsMkfs "/fd0/", pSbdFloppy
* .CE
* If a disk is new, use diskFormat() to format it.
*
* INTERNAL
* The fourth parameter passed to scsiPhysDevCreate() is now
* <reqSenseLength> (previously <selTimeout>).
*
*/

STATUS sysScsiConfig (void)
    {

#if FALSE  /* EXAMPLE CODE IS NOT COMPILED */

    UINT which;
    int  scsiId;
    char modeData [4];		/* array for floppy MODE SELECT data */
    SCSI_OPTIONS options;

    /*
     * NOTE: Either of the following global variables may be set or reset
     * from the VxWorks shell. Under 5.0, they should NOT both be set at the
     * same time, or output will be interleaved and hard to read!! They are
     * intended as an aid to trouble-shooting SCSI problems.
     */

    scsiDebug = FALSE;		/* enable SCSI debugging output */
    scsiIntsDebug = FALSE;	/* enable SCSI interrupt debugging output */

    /*
     * The following section of code provides sample configurations within
     * VxWorks of SCSI peripheral devices and VxWorks file systems. It 
     * should however be noted that the actual parameters provided to
     * scsiPhysDevCreate(), scsiBlkDevCreate(), dosFsDevInit() etc., are
     * highly dependent upon the user environment and should therefore be 
     * modified accordingly.
     */

    /*
     * HARD DRIVE CONFIGURATION
     *
     * In order to configure a hard disk and initialise both dosFs and rt11Fs
     * file systems, the following initialisation code will serve as an
     * example.
     */

    /* configure a SCSI hard disk at busId = 2, LUN = 0 */

    if ((pSpd20 = scsiPhysDevCreate (pSysScsiCtrl, 2, 0, 0, NONE, 0, 0, 0))
        == (SCSI_PHYS_DEV *) NULL)
	{
        printErr ("usrScsiConfig: scsiPhysDevCreate failed.\n",
			0, 0, 0, 0, 0, 0);
	}
    else
	{
	/* create block devices */

        if (((pSbd0 = scsiBlkDevCreate (pSpd20, 0x10000, 0)) == NULL)       ||
            ((pSbd1 = scsiBlkDevCreate (pSpd20, 0x10000, 0x10000)) == NULL) ||
            ((pSbd2 = scsiBlkDevCreate (pSpd20, 0, 0x20000)) == NULL))
	    {
    	    return (ERROR);
	    }

        if ((dosFsDevInit  ("/sd0/", pSbd0, NULL) == NULL) )
	    {
	    return (ERROR);
	    }

#ifdef INCLUDE_RT11FS
	    if ((rt11FsDevInit ("/sd1/", pSbd1, 0, 256, TRUE) == NULL) ||
	    (rt11FsDevInit ("/sd2/", pSbd2, 0, 256, TRUE) == NULL))
	    {
	    return (ERROR);
	    }
#endif
	}


    /* 
     * FLOPPY DRIVE CONFIGURATION
     * 
     * In order to configure a removable media floppy drive with a
     * dosFs file system, the following device specific code will serve
     * as an example. Note that some arguments like mode parameters are
     * highly devcie and vendor specific. Thus, the appropriate peripheral
     * hardware manual should be consulted.
     */

    /* configure floppy at busId = 3, LUN = 1 */

    if ((pSpd31 = scsiPhysDevCreate (pSysScsiCtrl, 3, 1, 0, NONE, 0, 0, 0))
	== (SCSI_PHYS_DEV *) NULL)
	{
        printErr ("usrScsiConfig: scsiPhysDevCreate failed.\n");
	return (ERROR);
	}

    /* 
     * Zero modeData array, then set byte 1 to "medium code" (0x1b). NOTE:
     * MODE SELECT data is highly device-specific. If your device requires
     * configuration via MODE SELECT, please consult the device's Programmer's
     * Reference for the relevant data format.
     */

    bzero (modeData, sizeof (modeData));
    modeData [1] = 0x1b;

    /* issue the MODE SELECT command to correctly configure floppy controller */

    scsiModeSelect (pSpd31, 1, 0, modeData, sizeof (modeData));

    /*
     * delete and re-create the SCSI_PHYS_DEV so that INQUIRY will return the
     * new device parameters, i.e., correct number of blocks
     */

    scsiPhysDevDelete (pSpd31);

    if ((pSpd31 = scsiPhysDevCreate (pSysScsiCtrl, 3, 1, 0, NONE, 0, 0, 0))
	== (SCSI_PHYS_DEV *) NULL)
	{
        printErr ("usrScsiConfig: scsiPhysDevCreate failed.\n");
	return (ERROR);
	}

    if ((pSbdFloppy = scsiBlkDevCreate (pSpd31, 0, 0)) == NULL)
	{
        printErr ("usrScsiConfig: scsiBlkDevCreate failed.\n");
	return (ERROR);
	}

    /*
     * Fill in the <blksPerTrack> (blocks (or sectors) per track) and <nHeads>
     * (number of heads) BLK_DEV fields, since it is impossible to ascertain
     * this information from the SCSI adapter card. This is important for
     * PC compatibility, primarily.
     */

    scsiBlkDevInit ((SCSI_BLK_DEV *) pSbdFloppy, 15, 2);

    /* Initialize as a dosFs device */

    /*
     * NOTE: pSbdFloppy is declared globally in case the following call
     * fails, in which case dosFsMkfs or dosFsDevInit can be
     * called (from the shell) with pSbdFloppy as an argument
     * (assuming pSbdFloppy != NULL)
     */

    if (dosFsDevInit ("/fd0/", pSbdFloppy, NULL) == NULL)
	{
        printErr ("usrScsiConfig: dosFsDevInit failed.\n");
	return (ERROR);
	}


    /* 
     * TAPE DRIVE CONFIGURATION
     * 
     * In order to configure a sequential access tape device and a tapeFs
     * file system, the following code will serve as an example. Note that
     * sequential access and tapeFs support are only available via SCSI-2.
     * To make sure that SCSI-2 is being used, check for the INCLUDE_SCSI2
     * macro definition in the BSP.
     *
     * The tape device does not support synchronous data transfers
     * or wide data transfers. Therefore, turn off the automatic configuration
     * of these features within VxWorks.
     */

    scsiId = 4;
    which = SCSI_SET_OPT_XFER_PARAMS | SCSI_SET_OPT_WIDE_PARAMS;

    options.maxOffset = SCSI_SYNC_XFER_ASYNC_OFFSET;
    options.minPeriod = SCSI_SYNC_XFER_ASYNC_PERIOD;
    options.xferWidth = SCSI_WIDE_XFER_SIZE_NARROW;

    if (scsiTargetOptionsSet (pSysScsiCtrl, scsiId, &options, which) == ERROR)
        {
        printf ("Could not set target option parameters\n");
        return (ERROR);
        }

    /* create SCSI physical device and sequential device */

    if ((pSpd40 = scsiPhysDevCreate (pSysScsiCtrl, scsiId, 0,0,NONE,0,0,0)) 
            == NULL)
        {
        printErr ("scsiPhysDevCreate failed.\n");
        return (ERROR);
        }

    if ((pSd0 = scsiSeqDevCreate (pSpd40)) == NULL)
        {
        printErr ("scsiSeqDevCreate failed.\n");
        return (ERROR);
        }

    /* configure a fixed block and rewind, tape file system */

    pTapeConfig = (TAPE_CONFIG *) calloc (sizeof(TAPE_CONFIG),1);
    pTapeConfig->rewind = TRUE;		/* rewind device */
    pTapeConfig->blkSize = 512;		/* fixed 512 byte block */ 

    if (tapeFsDevInit ("/tape1", pSd0, pTapeConfig) == NULL)
	{
	printErr ("tapeFsDevInit failed.\n");
	return (ERROR);
	}

#endif /*FALSE, END OF EXAMPLE CODE */

    return (OK);
    }

#endif /* INCLUDE_SCSI */
