/* sysSerial.c - Motorola MVME162 BSP serial device initialization */

/* Copyright 1984-1996 Wind River Systems, Inc. */
#include "copyright_wrs.h"

/*
modification history
--------------------
01g,05dec96,wlf  doc: cleanup.
01f,20aug96,dat  added sysSerialReset() (merge from scsi2)
01e,23oct95,jdi  doc: cleaned up and removed all NOMANUALs.
01d,21jun95,ms	 removed baudRate, channel init, (done in driver).
01c,20jun95,ms   fixed comments for mangen
01b,15jun95,ms	 updated for new serial driver.
01a,07mar95,ms	 created.
*/

#include "vxWorks.h"
#include "iv.h"
#include "intLib.h"
#include "mv162.h"
#include "config.h"
#include "sysLib.h"
#include "drv/sio/z8530Sio.h"

static Z8530_DUSART z8530Dusart1;
#ifdef	MVME162LX
static Z8530_DUSART z8530Dusart2;
#endif

/******************************************************************************
*
* sysSerialHwInit - initialize the BSP serial devices to a quiescent state
*
* This routine initializes the BSP serial device descriptors and puts the
* devices in a quiescent state.  It is called from sysHwInit() with
* interrupts locked.
*
* RETURNS: N/A
*
* SEE ALSO: sysHwInit()
*/ 

void sysSerialHwInit (void)
    {
    /* intialize the chips device descriptors */

    z8530Dusart1.portA.cr	= (char *) SERIAL_SCC_1+5;
    z8530Dusart1.portA.dr	= (char *) SERIAL_SCC_1+7;
    z8530Dusart1.portA.baudFreq = BAUD_CLK_FREQ;
    z8530Dusart1.portA.writeReg11 = SCC_WR11_RX_BR_GEN | SCC_WR11_TX_BR_GEN |
			 		SCC_WR11_OUT_BR_GEN;
    z8530Dusart1.portA.writeReg14 = SCC_WR14_BR_EN | SCC_WR14_BR_SRC |
			 		SCC_WR14_SRC_BR;
    z8530Dusart1.portA.intType	= SCC_WR9_VIS;
    z8530Dusart1.portA.intVec	= INT_VEC_SCC_1;

    z8530Dusart1.portB.cr	= (char *) SERIAL_SCC_1+1;
    z8530Dusart1.portB.dr	= (char *) SERIAL_SCC_1+3;
    z8530Dusart1.portB.baudFreq = BAUD_CLK_FREQ;
    z8530Dusart1.portB.writeReg11 = SCC_WR11_RX_BR_GEN | SCC_WR11_TX_BR_GEN |
			 		SCC_WR11_OUT_BR_GEN;
    z8530Dusart1.portB.writeReg14 = SCC_WR14_BR_EN | SCC_WR14_BR_SRC |
					SCC_WR14_SRC_BR;
    z8530Dusart1.portB.intType	= SCC_WR9_VIS;
    z8530Dusart1.portB.intVec	= INT_VEC_SCC_1;

#ifdef	MVME162LX
    z8530Dusart2.portA.cr	= (char *) SERIAL_SCC_2+5;
    z8530Dusart2.portA.dr	= (char *) SERIAL_SCC_2+7;
    z8530Dusart2.portA.baudFreq = BAUD_CLK_FREQ;
    z8530Dusart2.portA.writeReg11 = SCC_WR11_RX_BR_GEN | SCC_WR11_TX_BR_GEN |
			 		SCC_WR11_OUT_BR_GEN;
    z8530Dusart2.portA.writeReg14 = SCC_WR14_BR_EN | SCC_WR14_BR_SRC |
			 		SCC_WR14_SRC_BR;
    z8530Dusart2.portA.intType	= SCC_WR9_VIS;
    z8530Dusart2.portA.intVec	= INT_VEC_SCC_2;

    z8530Dusart2.portB.cr	= (char *) SERIAL_SCC_2+1;
    z8530Dusart2.portB.dr	= (char *) SERIAL_SCC_2+3;
    z8530Dusart2.portB.baudFreq = BAUD_CLK_FREQ;
    z8530Dusart2.portB.writeReg11 = SCC_WR11_RX_BR_GEN | SCC_WR11_TX_BR_GEN |
			 		SCC_WR11_OUT_BR_GEN;
    z8530Dusart2.portB.writeReg14 = SCC_WR14_BR_EN | SCC_WR14_BR_SRC |
					SCC_WR14_SRC_BR;
    z8530Dusart2.portB.intType	= SCC_WR9_VIS;
    z8530Dusart2.portB.intVec	= INT_VEC_SCC_2;
#endif	/* MVME162LX */

    /* reset the chip */

    z8530DevInit (&z8530Dusart1);

#ifdef	MVME162LX
    z8530DevInit (&z8530Dusart2);
#endif	/* MVME162LX */
    }

/******************************************************************************
*
* sysSerialHwInit2 - connect BSP serial device interrupts
*
* This routine connects the BSP serial device interrupts.  It is called from
* sysHwInit2().  
* 
* Serial device interrupts cannot be connected in sysSerialHwInit() because
* the kernel memory allocator is not initialized at that point, and
* intConnect() calls malloc().
*
* RETURNS: N/A
*
* SEE ALSO: sysHwInit2()
*/ 

void sysSerialHwInit2 (void)
    {
    (void) intConnect (INUM_TO_IVEC (INT_VEC_SCC_A_WR(INT_VEC_SCC_1)),
                           z8530IntWr, (int)sysSerialChanGet(0));
    (void) intConnect (INUM_TO_IVEC (INT_VEC_SCC_A_EX(INT_VEC_SCC_1)),
                           z8530IntEx, (int)sysSerialChanGet(0));
    (void) intConnect (INUM_TO_IVEC (INT_VEC_SCC_A_RD(INT_VEC_SCC_1)),
                           z8530IntRd, (int)sysSerialChanGet(0));
    (void) intConnect (INUM_TO_IVEC (INT_VEC_SCC_A_SP(INT_VEC_SCC_1)),
                           z8530IntEx, (int)sysSerialChanGet(0));

    (void) intConnect (INUM_TO_IVEC (INT_VEC_SCC_B_WR(INT_VEC_SCC_1)),
                           z8530IntWr, (int)sysSerialChanGet(1));
    (void) intConnect (INUM_TO_IVEC (INT_VEC_SCC_B_EX(INT_VEC_SCC_1)),
                           z8530IntEx, (int)sysSerialChanGet(1));
    (void) intConnect (INUM_TO_IVEC (INT_VEC_SCC_B_RD(INT_VEC_SCC_1)),
                           z8530IntRd, (int)sysSerialChanGet(1));
    (void) intConnect (INUM_TO_IVEC (INT_VEC_SCC_B_SP(INT_VEC_SCC_1)),
                           z8530IntEx, (int)sysSerialChanGet(1));

#ifdef	MVME162LX
    (void) intConnect (INUM_TO_IVEC (INT_VEC_SCC_A_WR(INT_VEC_SCC_2)),
                           z8530IntWr, (int)sysSerialChanGet(2));
    (void) intConnect (INUM_TO_IVEC (INT_VEC_SCC_A_EX(INT_VEC_SCC_2)),
                           z8530IntEx, (int)sysSerialChanGet(2));
    (void) intConnect (INUM_TO_IVEC (INT_VEC_SCC_A_RD(INT_VEC_SCC_2)),
                           z8530IntRd, (int)sysSerialChanGet(2));
    (void) intConnect (INUM_TO_IVEC (INT_VEC_SCC_A_SP(INT_VEC_SCC_2)),
                           z8530IntEx, (int)sysSerialChanGet(2));

    (void) intConnect (INUM_TO_IVEC (INT_VEC_SCC_B_WR(INT_VEC_SCC_2)),
                           z8530IntWr, (int)sysSerialChanGet(3));
    (void) intConnect (INUM_TO_IVEC (INT_VEC_SCC_B_EX(INT_VEC_SCC_2)),
                           z8530IntEx, (int)sysSerialChanGet(3));
    (void) intConnect (INUM_TO_IVEC (INT_VEC_SCC_B_RD(INT_VEC_SCC_2)),
                           z8530IntRd, (int)sysSerialChanGet(3));
    (void) intConnect (INUM_TO_IVEC (INT_VEC_SCC_B_SP(INT_VEC_SCC_2)),
                           z8530IntEx, (int)sysSerialChanGet(3));
#endif	/* MVME162LX */

    /* enable serial interrupts */

    *MCC_SCC_ICR = SCC_ICR_IEN | SCC_IRQ_LEVEL;
    }

/******************************************************************************
*
* sysSerialChanGet - get the SIO_CHAN device associated with a serial channel
*
* This routine returns a pointer to the SIO_CHAN device associated
* with a specified serial channel.  It is called by usrRoot() to obtain 
* pointers when creating the system serial devices, `/tyCo/x'.  It
* is also used by the WDB agent to locate its serial channel.
*
* RETURNS: A pointer to the SIO_CHAN structure for the channel, or ERROR
* if the channel is invalid.
*/

SIO_CHAN * sysSerialChanGet
    (
    int channel		/* serial channel */
    )
    {
    switch (channel)
	{
	case 0:
	    return ((SIO_CHAN *)&z8530Dusart1.portA);
	case 1:
	    return ((SIO_CHAN *)&z8530Dusart1.portB);

#ifdef	MVME162LX
	case 2:
	    return ((SIO_CHAN *)&z8530Dusart2.portA);
	case 3:
	    return ((SIO_CHAN *)&z8530Dusart2.portB);
#endif	/* MVME162LX */

	default:
	    return ((SIO_CHAN *)ERROR);
	}
    }

/******************************************************************************
*
* sysSerialReset - reset serial devices to a quiescent state
*
* This routine resets all serial devices to a quiescent state to prevent
* them from generating interrupts or performing DMA cycles.
*
* This rotuine is called by sysToMonitor(), thus shutting down the system 
* before there is a transfer to boot ROMs.
*
* RETURNS: N/A
*
* SEE ALSO: sysToMonitor()
*/

void sysSerialReset (void)
    {
    z8530DevInit (&z8530Dusart1);

#ifdef	MVME162LX
    z8530DevInit (&z8530Dusart2);
#endif	/* MVME162LX */
    }
